unit acQBEasyTableMetaProvider;

interface

uses Classes, Contnrs, DB, acQBBase, acAST, EasyTable, acQBStrings, SysUtils;

type
  TacEasyTableMetadataProvider = class(TacBaseMetadataProvider)
  private
    fConnection: TEasyDatabase;
    procedure SetConnection(const Value: TEasyDatabase);
  protected
    function IsConnected: boolean; override;
    procedure DoConnect; override;
    procedure DoDisconnect; override;

    procedure CheckConnectionSet; override;

    function PrepareSQLDatasetInternal(const ASQL: WideString): TDataset; override;
    procedure ExecSQLInternal(const ASQL: WideString); override;

    procedure Notification(AComponent: TComponent; Operation: TOperation);
      override;
  public
    destructor Destroy; override;

    procedure LoadMetadataObjects(ASyntaxProvider: TacBaseSyntaxProvider;
      AMetadataFilter: TacMetadataFilter;
      AMetadataContainer: TacMetadataContainer; ADatabase: TSQLQualifiedName);
      override;

    function GetConnectionDescription: WideString; override;
  published
    property Database: TEasyDatabase read fConnection write SetConnection;
  end;

implementation

uses Variants;

{ TacEasyTableMetadataProvider }

procedure TacEasyTableMetadataProvider.CheckConnectionSet;
begin
  if Database = nil then
  begin
    raise Exception.CreateFmt(QueryBuilderLocalizer.ReadString(
      'acQBNoConnectionObject', acQBNoConnectionObject), ['Database'])
  end;
end;

destructor TacEasyTableMetadataProvider.Destroy;
begin
  Database := nil;
  inherited;
end;

procedure TacEasyTableMetadataProvider.DoConnect;
begin
  inherited;
  CheckConnectionSet;
  Database.Open;
end;

procedure TacEasyTableMetadataProvider.DoDisconnect;
begin
  inherited;
  CheckConnectionSet;
  Database.Close;
end;

function TacEasyTableMetadataProvider.PrepareSQLDatasetInternal(
  const ASQL: WideString): TDataset;
begin
  Result := TEasyQuery.Create(nil);
  TEasyQuery(Result).DatabaseName := Database.DatabaseName;
  TEasyQuery(Result).SQL.Text := ASQL;
end;

procedure TacEasyTableMetadataProvider.ExecSQLInternal(
  const ASQL: WideString);
var
  q: TEasyQuery;
begin
  q := TEasyQuery.Create(nil);
  try
    q.DatabaseName := Database.DatabaseName;
    q.SQL.Text := ASQL;
    q.ExecSQL;
  finally
    q.Free;
  end;
end;

function TacEasyTableMetadataProvider.GetConnectionDescription: WideString;
begin
  Result := 'EasyTable Connection';
end;

function TacEasyTableMetadataProvider.IsConnected: boolean;
begin
  if Database <> nil then
  begin
    Result := Database.Connected
  end
  else begin
    Result := false
  end;
end;

procedure TacEasyTableMetadataProvider.LoadMetadataObjects(
  ASyntaxProvider: TacBaseSyntaxProvider; AMetadataFilter: TacMetadataFilter;
  AMetadataContainer: TacMetadataContainer; ADatabase: TSQLQualifiedName);
var
  mof: TacMetadataObjectsFetcherFromStringLists;
begin
  if IsQualifiedNameEmpty(ADatabase) then
  begin
    Database.Open;
    mof := TacMetadataObjectsFetcherFromStringLists.Create(ASyntaxProvider, Self);
    try
      Database.GetTablesList(mof.UserObjects);
      mof.DefaultObjectClass := TacMetadataTable;
      mof.LoadMetadata(AMetadataFilter, AMetadataContainer, ADatabase);
    finally
      mof.Free;
    end;
  end
  else begin
    inherited
  end;
end;

procedure TacEasyTableMetadataProvider.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  if (Operation = opRemove) and (AComponent = Database) then
  begin
    fConnection := nil
  end;
  inherited;
end;

procedure TacEasyTableMetadataProvider.SetConnection(
  const Value: TEasyDatabase);
begin
  if fConnection <> Value then
  begin
    if fConnection <> nil then
    begin
      fConnection.RemoveFreeNotification(Self)
    end;
    fConnection := Value;
    if fConnection <> nil then
    begin
      fConnection.FreeNotification(Self)
    end;
  end;
end;

end.
